/*	
 *	Pollin Electronic GmbH
 *	Max-Pollin-Strae 1			
 *	85104 Pfrring 
 *	http://www.pollin.de
 *
 *	Project:	RAPITUS SAMxS Evaluationboard 
 *  Filename:	buttons_adc.c 
 *
 *  Author:		Leonhard Hesse
 *
 *  Copyright (c) 2016 Pollin Electronic GmbH. All rights reserved.
 *
 *  Bitte beachten Sie, dass dieses Programm nur uneingeschrnkt fr nicht kommerzielle Zwecke genutzt 
 *  werden darf. Ich bernehme keine Garantie fr die einwandfreie Funktion dieses Programmes und ber- 
 *  nehme dadurch auch keine Haftung fr Folgeschden die Aufgrund der Benutzung dieser Daten entstehen. 
 *  Dieses gilt auch dann, wenn ich ber einen solchen Schaden unterrichtet worden bin. 
 */

 #include "asf.h"
 #include "buttons_adc.h"
 #include "lcd.h"

 uint8_t debounce = 0;

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		adc_setup:	ACD Einstellungen vornehmen und ADC starten.
 -------------------------------------------------------------------------------------------------------------------- */
 void adc_setup(void)
 {
	 pmc_enable_periph_clk(ID_ADC);								// enable peripheral clock for ADC
	 adc_init(ADC, sysclk_get_main_hz(), ADC_CLOCK, 8);			// initialize ADC-Clock 6,4 MHz
	 adc_set_resolution(ADC, ADC_MR_LOWRES_BITS_12);			// set the ADC-Resolution to 12-Bit
	 adc_configure_timing(ADC, 1, ADC_SETTLING_TIME_3, 1);		// set ADC-Tracking to 17 periods of ADCCLK
	 adc_configure_trigger(ADC, ADC_TRIG_SW, 0);				// enable software conversation
	 adc_enable_channel(ADC, ADC_CHANNEL_8);					// enable ADC-Channel 8
	 NVIC_EnableIRQ(ADC_IRQn);									// enable ADC-Interrupt in the NVIC
	 adc_start(ADC);											// start the ADC
	 adc_enable_interrupt(ADC, ADC_IMR_DRDY);					// enable Interrupt "data ready"
 }

 /* --------------------------------------------------------------------------------------------------------------------
	ADC Interrupt Handler	ADC_Handler: Interpetation der Messwerte des ADC
							Wird aufgerufen, wenn der Interrupt des ADCs ausgelt wird.
							Prft den aktuellen Messwert und ordnet die entspechende Funktion zu.
							Lst bei einem eindeutigen Pegel die Methode manageMenu aus und bergibt dieser
							einen Wert, der die gedrckte Taste reprsentiert.

	Variablen:				result:		Reprsentiert den aktuellen Wert des ADCs
							debounce:	Reprsentiert eine Zhlervariable um eine kurze Verzgerung zu generieren.
										Diese ist ntig, um auf einen eindeutigen Pegel zu warten und um mehrfaches
										auslen bei einem Tastendruck zu verhindern.
 -------------------------------------------------------------------------------------------------------------------- */

 void ADC_Handler(void)
 {
	 // Check the ADC conversion status
	 if ((adc_get_status(ADC) & ADC_ISR_DRDY) == ADC_ISR_DRDY)
	 {
		 // Get latest digital data value from ADC so it can be used by application
		 uint32_t result = adc_get_latest_value(ADC);

		 /* --------------- Button 1 --------------- */
		 if (result > 1132 && result < 1166)
		 {
			 if (debounce < DEBOUNC)
			 {
				 debounce++;
			 }
			 else
			 {
				 if (debounce == DEBOUNC)
				 {
					 debounce++;
					 manageMenu(BUTTON_UP);
				 }
			 }
		 }

		 /* --------------- Button 2 --------------- */
		 if (result > 1292 && result < 1327)
		 {
			 if (debounce < DEBOUNC)
			 {
				 debounce++;
			 }
			 else
			 {
				 if (debounce == DEBOUNC)
				 {
					 debounce++;
					 manageMenu(BUTTON_LEFT);
				 }
			 }
		 }

		 /* --------------- Button 3 --------------- */
		 if (result > 1001 && result < 1031)
		 {
			 if (debounce < DEBOUNC)
			 {
				 debounce++;
			 }
			 else
			 {
				 if (debounce == DEBOUNC)
				 {
					 debounce++;
					 manageMenu(BUTTON_RETURN);
				 }
			 }
		 }

		 /* --------------- Button 4 --------------- */
		 if (result > 1214 && result < 1249)
		 {
			 if (debounce < DEBOUNC)
			 {
				 debounce++;
			 }
			 else
			 {
				 if (debounce == DEBOUNC)
				 {
					 debounce++;
					 manageMenu(BUTTON_RIGHT);
				 }
			 }
		 }

		 /* --------------- Button 5 --------------- */
		 if (result > 1068 && result < 1100)
		 {
			 if (debounce < DEBOUNC)
			 {
				 debounce++;
			 }
			 else
			 {
				 if (debounce == DEBOUNC)
				 {
					 debounce++;
					 manageMenu(BUTTON_DOWN);
				 }
			 }
		 }

		 /* ---------- No Button pressed ---------- */
		 if (result > 3000)
		 {
			 debounce = 0;
		 }
	 }
 }