/*	
 *	Pollin Electronic GmbH
 *	Max-Pollin-Strae 1			
 *	85104 Pfrring 
 *	http://www.pollin.de
 *
 *	Project:	RAPITUS SAMxS Evaluationboard 
 *  Filename:	enc.c 
 *
 *  Author:		Leonhard Hesse
 *
 *  Copyright (c) 2016 Pollin Electronic GmbH. All rights reserved.
 *
 *  Bitte beachten Sie, dass dieses Programm nur uneingeschrnkt fr nicht kommerzielle Zwecke genutzt 
 *  werden darf. Ich bernehme keine Garantie fr die einwandfreie Funktion dieses Programmes und ber- 
 *  nehme dadurch auch keine Haftung fr Folgeschden die Aufgrund der Benutzung dieser Daten entstehen. 
 *  Dieses gilt auch dann, wenn ich ber einen solchen Schaden unterrichtet worden bin. 
 *
 *	Die Grundlage fr diesen Code stammt von Simon Schulz (avr@auctionant.de) und wurde fr die Verwendung
 *	mit dem SAM4S modifiziert. 
 *
 *	This program is free software. You can redistribute it and/or modify it under the terms of the 
 *	GNU General Public License as published by the Free Software Foundation. Either version 2 of the License, 
 *	or (at your option) any later version. This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY. Without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *	PURPOSE. See the GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License along with this program. If not, 
 *	write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 *	http://www.gnu.de/gpl-ger.html
 */

 #include "asf.h"
 #include "mySpi.h"
 #include "enc.h"

 uint8_t enc_bankSelect;
 uint16_t enc_nextPacket;
 uint8_t test_rx_data;
 uint8_t enc_ID = 0;

 void enc_select(void)
 {
	gpio_set_pin_low(SPI_NPCS3_GPIO);
 }

 void enc_deselect(void)
 {
	gpio_set_pin_high(SPI_NPCS3_GPIO);
	delay_us(5);
 }

 uint8_t enc_read(uint8_t address)
 {
	enc_setBank(address);	
	return enc_readCommand(ENC_READ_CTRL_REG, address);
 }

 uint8_t enc_readCommand(uint8_t opCode, uint8_t address)
 {
	enc_select();
	mySpi_rw(SPI, opCode | (address & ADDR_MASK), SPI_PCS_ENC, SPI_LAST_BYTE_FALSE);		// write read-command to ENC
	if(address & 0x80)		// MAC | MII Register -> nedd dummy Byte
	{
		mySpi_rw(SPI, 0x00, SPI_PCS_ENC, SPI_LAST_BYTE_FALSE);
	}
	return mySpi_rw(SPI, 0x00, SPI_PCS_ENC, SPI_LAST_BYTE_TRUE);	// send dummy to read
	enc_deselect();
 }

 void enc_setBank(uint8_t address)
 {
	 // set the bank (if needed)
	 if((address & BANK_MASK) != enc_bankSelect)
	 {
		 enc_select();
		 enc_writeCommand(ENC_BIT_FIELD_CLR, ECON1, (ECON1_BSEL1|ECON1_BSEL0));
		 enc_writeCommand(ENC_BIT_FIELD_SET, ECON1, (address & BANK_MASK)>>5);
		 enc_bankSelect = (address & BANK_MASK);
		 enc_deselect();
	 }
 }

 void enc_reset(void)
 {
	enc_select();
	mySpi_rw(SPI, ENC_SOFT_RESET, SPI_PCS_ENC, SPI_LAST_BYTE_TRUE);
	enc_deselect();
 }

 void enc_write(uint8_t address, uint8_t data)
 {
	enc_setBank(address);
	enc_writeCommand(ENC_WRITE_CTRL_REG, address, data);
 }

 void enc_writeCommand (uint8_t opCode, uint8_t address, uint8_t data)
 {
	enc_select();
	mySpi_rw(SPI, opCode | (address & ADDR_MASK), SPI_PCS_ENC, SPI_LAST_BYTE_FALSE);
	mySpi_rw(SPI, data, SPI_PCS_ENC, SPI_LAST_BYTE_TRUE);
	enc_deselect();
 }

 void enc_writePhy(uint8_t address, uint16_t data)
 {
	 // set the PHY register address
	 enc_write(MIREGADR, address);
	 
	 // write the PHY data
	 enc_write(MIWRL, LO8(data));
	 enc_write(MIWRH, HI8(data));

	 // wait until the PHY write completes
	 delay_us(10);
	 while(enc_read(MISTAT) & MISTAT_BUSY);
	 delay_ms(20);
 }

 void enc_read_buf(uint8_t *buf, unsigned int len)
 {
	 enc_select();
	 mySpi_rw(SPI, ENC_READ_BUF_MEM, SPI_PCS_ENC, SPI_LAST_BYTE_FALSE);
	 for(; len > 0; len--, buf++ ) {
		 *buf = mySpi_rw(SPI, 0x00, SPI_PCS_ENC, SPI_LAST_BYTE_FALSE);
	 }
	 enc_deselect();
 }

 void enc_write_buf(uint8_t *buf, unsigned int len)
 {
	 enc_select();
	 mySpi_rw(SPI, ENC_WRITE_BUF_MEM, SPI_PCS_ENC, SPI_LAST_BYTE_FALSE);
	 for(; len > 0; len--, buf++ ) {
		 mySpi_rw(SPI, *buf, SPI_PCS_ENC, SPI_LAST_BYTE_FALSE);
	 }
	 enc_deselect();
 }

 void enc_send_packet(unsigned int len, uint8_t *buf)
 {
	 eth.no_reset = 1;
	 uint8_t ctrl = 0;
	 unsigned int ms = 100;

	 // wait up to 100 ms for the previos tx to finish
	 while(ms--) 
	 {
		 if( !(enc_read(ECON1) & ECON1_TXRTS)) break;
		 delay_ms(1);
	 }

	 // errata #12: reset tx logic
	 enc_writeCommand(ENC_BIT_FIELD_SET, ECON1, ECON1_TXRTS);
	 enc_writeCommand(ENC_BIT_FIELD_CLR, ECON1, ECON1_TXRTS);

	 // setup write pointer
	 enc_write(EWRPTL, LO8(ENC_TX_BUFFER_START));
	 enc_write(EWRPTH, HI8(ENC_TX_BUFFER_START));
	 
	 // end pointer (points to last byte) to start + len
	 enc_write(ETXNDL, LO8(ENC_TX_BUFFER_START+len));
	 enc_write(ETXNDH, HI8(ENC_TX_BUFFER_START+len));

	 // enc requires 1 control byte before the package
	 enc_write_buf(&ctrl, 1);

	 // copy packet to enc buffer
	 enc_write_buf(buf, len);

	 // clear TXIF flag
	 enc_writeCommand(ENC_BIT_FIELD_CLR, EIR, EIR_TXIF); 

	 // start transmission by setting the TXRTS bit
	 enc_writeCommand(ENC_BIT_FIELD_SET, ECON1, ECON1_TXRTS);
 }

 unsigned int enc_receive_packet( unsigned int bufsize, uint8_t *buf)
 {
	 uint8_t rxheader[6];
	 unsigned int len, status;
	 uint8_t u;

	 // check rx packet counter
	 u = enc_read(EPKTCNT);
	 if(u == 0) {
		 // packetcounter is 0, there is nothing to receive, go back
		 return 0;
	 }

	 //set read pointer to next packet
	 enc_write(ERDPTL, LO8(enc_nextPacket));
	 enc_write(ERDPTH, HI8(enc_nextPacket));

	 // read enc rx packet header
	 enc_read_buf(rxheader, sizeof(rxheader));
	 enc_nextPacket		 =             rxheader[0];
	 enc_nextPacket		|= ((unsigned)(rxheader[1])) << 8;
	 len                 =             rxheader[2];
	 len                |= ((unsigned)(rxheader[3])) << 8;
	 status              =             rxheader[4];
	 status             |= ((unsigned)(rxheader[5])) << 8;

	 // added by Sjors: reset the ENC when needed
	 // If the receive OK bit is not 1 or the zero bit is not zero, or the packet is larger then the buffer, reset the enc chip and SPI
	 if ((!(status & (1<<7))) || (status & 0x8000) || (len > 1518))
	 {
		 enc_init();
	 }

	 // skip the checksum (4 bytes) at the end of the buffer
	 len -= 4;

	 // if the application buffer is to small, we just truncate
	 if( len > bufsize ) len = bufsize;

	 // now read the packet data into buffer
	 enc_read_buf(buf, len);

	 // adjust the ERXRDPT pointer (= free this packet in rx buffer)
	 if(enc_nextPacket-1 > ENC_RX_BUFFER_END || enc_nextPacket-1 < ENC_RX_BUFFER_START ) 
	 {
		 enc_write(ERXRDPTL, LO8(ENC_RX_BUFFER_END));
		 enc_write(ERXRDPTH, HI8(ENC_RX_BUFFER_END));
	 } else 
	 {
		 enc_write(ERXRDPTL, LO8(enc_nextPacket-1));
		 enc_write(ERXRDPTH, HI8(enc_nextPacket-1));
	 }

	 // trigger a decrement of the rx packet counter
	 // this will clear PKTIF if EPKTCNT reaches 0
	 enc_writeCommand(ENC_BIT_FIELD_SET, ECON2, ECON2_PKTDEC);

	 // return number of bytes written to the buffer
	 return len;
 }

 void enc_init(void)
 {
	enc_reset();									// reset the enc Controller -> send 0xFF

	while (!enc_read(ESTAT) & ESTAT_CLKRDY)			//check if enc is ready for init

	enc_ID = enc_read(EREVID);

	enc_writeCommand(ENC_BIT_FIELD_CLR, ECON1, ECON1_RXEN);

	//rx-buffer
	enc_write(ERXSTL, LO8(ENC_RX_BUFFER_START));
	enc_write(ERXSTH, HI8(ENC_RX_BUFFER_START));
	enc_write(ERXNDL, LO8(ENC_RX_BUFFER_END));
	enc_write(ERXNDH, HI8(ENC_RX_BUFFER_END));

	// tx-buffer
	enc_write(ETXSTL, LO8(ENC_TX_BUFFER_START));
	enc_write(ETXSTH, HI8(ENC_TX_BUFFER_START));
	enc_write(ETXNDL, LO8(ENC_TX_BUFFER_END));
	enc_write(ETXNDH, HI8(ENC_TX_BUFFER_END));
	
	// bring MAC out of reset
	enc_write(MACON2, 0x00);
	
	// enable MAC receive
	enc_write(MACON1, MACON1_MARXEN|MACON1_TXPAUS|MACON1_RXPAUS);	

	// mac auto padding of small packets, add crc, frame length check, half duplex
	enc_write(MACON3, MACON3_PADCFG0|MACON3_TXCRCEN|MACON3_FRMLNEN);	
	
	// max framelength 1518
	enc_write(MAMXFLL, LO8(MAX_FRAMELEN));
	enc_write(MAMXFLH, HI8(MAX_FRAMELEN));

	// set inter-frame gap (back-to-back)
	enc_write(MABBIPG, 0x12);

	// set inter-frame gap (non-back-to-back)
	enc_write(MAIPGL, 0x12);
	enc_write(MAIPGH, 0x0C);

	// write MAC address
	// NOTE: MAC address in ENC28J60 is byte-backward
	enc_write(MAADR6, ENC_MAC1);
	enc_write(MAADR5, ENC_MAC2);
	enc_write(MAADR4, ENC_MAC3);
	enc_write(MAADR3, ENC_MAC4);
	enc_write(MAADR2, ENC_MAC5);
	enc_write(MAADR1, ENC_MAC6);

	// disable CLKOUT pin
	enc_write(ECOCON, 0x00);

	delay_ms(20);

	// no loopback of transmitted frames
	enc_writePhy(PHCON2, PHCON2_HDLDIS);

	// LED config LEDA->on LEDB RX&TX activity
	enc_writePhy(PHLCON, 0x0872);	

	/* configure the enc interrupt sources - 1<<INTIE, 1<<PKTIE */
	enc_write(EIE, 0xC0);

	// enable receive
	enc_nextPacket = ENC_RX_BUFFER_START;
	enc_writeCommand(ENC_BIT_FIELD_SET, ECON1, ECON1_RXEN);

	/* enable interrupt for ENC INT-Pin  */
	pio_enable_interrupt(PIOA, PIO_PA0);
	NVIC_EnableIRQ(PIOA_IRQn);
 }	
