/*	
 *	Pollin Electronic GmbH
 *	Max-Pollin-Strae 1			
 *	85104 Pfrring 
 *	http://www.pollin.de
 *
 *	Project:	RAPITUS SAMxS Evaluationboard 
 *  Filename:	lcd.c 
 *
 *  Author:		Leonhard Hesse
 *
 *  Copyright (c) 2016 Pollin Electronic GmbH. All rights reserved.
 *
 *  Bitte beachten Sie, dass dieses Programm nur uneingeschrnkt fr nicht kommerzielle Zwecke genutzt 
 *  werden darf. Ich bernehme keine Garantie fr die einwandfreie Funktion dieses Programmes und ber- 
 *  nehme dadurch auch keine Haftung fr Folgeschden die Aufgrund der Benutzung dieser Daten entstehen. 
 *  Dieses gilt auch dann, wenn ich ber einen solchen Schaden unterrichtet worden bin. 
 */

 #include "asf.h"
 #include "string.h"
 #include "lcd.h"
 #include "buttons_adc.h"
 #include "uart.h"

 char timeSetting_time_str[16];
 char timeSetting_date_str[16];
 char timeSetting_week_str[16];

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		LCD_Init:	Initialisiert das LCD-Display
								Die Initialisierung dieses LCD-Displays besteht aus mehreren Schritten. Der genaue 
								Vorgang ist im Datenblatt des Displays beschrieben. Dieser Code ist fr das
								LCD-Modul TC1604A-05 geschrieben worden. Ein 4-zeiliges STN-Display. (Art.Nr. 120 689)
 -------------------------------------------------------------------------------------------------------------------- */
 void LCD_Init(void)
 {
	 lcd_welcomeDelay = 0;

	 ioport_set_port_level(IOPORT_PIOC,LCD_MASK_E,RESET);
	 ioport_set_port_level(IOPORT_PIOC,LCD_MASK_RS,RESET);
	 ioport_set_port_level(IOPORT_PIOC,LCD_MASK_DB,RESET);

	 delay_ms(LCD_BOOT_DEL_MS);													// Boot delay

	 ioport_set_port_level(IOPORT_PIOC,LCD_MASK_DB,(LCD_INIT_ATT<<5));			// LCD attention! repeat 3 times
	 trigger_LCD_E();
	 delay_ms(LCD_INIT_ATT_DEL_MS);

	 trigger_LCD_E();
	 delay_us(LCD_INIT_ATT_DEL_US);

	 trigger_LCD_E();
	 delay_ms(LCD_INIT_ATT_DEL_MS);

	 ioport_set_port_level(IOPORT_PIOC,LCD_MASK_DB,RESET);
	 ioport_set_port_level(IOPORT_PIOC,LCD_MASK_DB,(LCD_INIT_FUNC<<5));			// 8 Bit, 2/16, 5x8	
	 trigger_LCD_E();
	 delay_us(LCD_INIT_COM_DEL_US);

	 ioport_set_port_level(IOPORT_PIOC,LCD_MASK_DB,RESET);
	 ioport_set_port_level(IOPORT_PIOC,LCD_MASK_DB,(LCD_INIT_OFF<<5));			// display off
	 trigger_LCD_E();
	 delay_us(LCD_INIT_COM_DEL_US);

	 ioport_set_port_level(IOPORT_PIOC,LCD_MASK_DB,RESET);
	 ioport_set_port_level(IOPORT_PIOC,LCD_MASK_DB,(LCD_INIT_ON<<5));			// display on
	 trigger_LCD_E();
	 delay_us(LCD_INIT_COM_DEL_US);

	 ioport_set_port_level(IOPORT_PIOC,LCD_MASK_DB,RESET);
	 ioport_set_port_level(IOPORT_PIOC,LCD_MASK_DB,(LCD_INIT_MODE<<5));			// entry mode
	 trigger_LCD_E();
	 delay_us(LCD_INIT_COM_DEL_US);

	 LCD_Clear();
 }

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		lcd_Clear:	Lscht alle auf dem Display angezeigten Zeichen und setzt den Cursor zurck.
 -------------------------------------------------------------------------------------------------------------------- */
 void LCD_Clear(void)
 {
 	 ioport_set_port_level(IOPORT_PIOC,LCD_MASK_RS,RESET);
 	 ioport_set_port_level(IOPORT_PIOC,LCD_MASK_DB,RESET);
 	 ioport_set_port_level(IOPORT_PIOC,LCD_MASK_DB,(LCD_INIT_CLEAR<<5));		// clear LCD
 	 trigger_LCD_E();
 	 delay_ms(LCD_INIT_CLEAR_DEL_MS);
 }

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		lcd_setCursor:	Aktualisiert die Position des Cursors.

	Variablen:		x:	X-Position des Cursors, Spalt.
					y:	Y-Position des Cursors, Zeile.			
 -------------------------------------------------------------------------------------------------------------------- */
 void lcd_setCursor( uint8_t x, uint8_t y )
 {
	 uint8_t data;
	 
	 switch (y)
	 {
		 case 1:    // Line 1
			data = LCD_SET_DDADR + LCD_DDADR_LINE1 + x;
		 break;
		 
		 case 2:    // Line 2
			data = LCD_SET_DDADR + LCD_DDADR_LINE2 + x;
		 break;
		 
		 case 3:    // Line 3
			data = LCD_SET_DDADR + LCD_DDADR_LINE3 + x;
		 break;
		 
		 case 4:    // Line 4
			data = LCD_SET_DDADR + LCD_DDADR_LINE4 + x;
		 break;
		 
		 default:
		 return;
	 }
	 ioport_set_port_level(IOPORT_PIOC,LCD_MASK_RS,RESET);
	 ioport_set_port_level(IOPORT_PIOC,LCD_MASK_DB,RESET);
	 ioport_set_port_level(IOPORT_PIOC,LCD_MASK_DB,((RESET) | (data<<5)));
	 trigger_LCD_E();
	 delay_us(LCD_INIT_COM_DEL_US);
 }

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		trigger_LCD_E:	Schaltet den LCD-Enable Pin fr 1us ein 
					Der LCD-Enable Pin wird benutzt, um dem Display den Empfang von Daten zu signalisieren.
 -------------------------------------------------------------------------------------------------------------------- */
 void trigger_LCD_E(void)
 {
	 gpio_toggle_pin(LCD_E);
	 delay_us(LCD_TRIG_E_DEL_US);
	 gpio_toggle_pin(LCD_E);
 }

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		Write_string_lcd:	Sendet einen String an das Display.
 -------------------------------------------------------------------------------------------------------------------- */
 void Write_string_lcd(const char * s)
 {
	 ioport_set_port_level(IOPORT_PIOC,LCD_MASK_RS,(LCD_RS_DATA<<2));		// RS auf 1 setzen
	 while(*s)
	 Write_char_lcd(*s++);
 }

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		Write_char_lcd:		Sendet ein Zeichen an das Display.
 -------------------------------------------------------------------------------------------------------------------- */
 void Write_char_lcd(unsigned char c)
 {
	 ioport_set_port_level(IOPORT_PIOC,LCD_MASK_RS,(LCD_RS_DATA<<2));		// RS auf 1 setzen
	 ioport_set_port_level(IOPORT_PIOC,LCD_MASK_DB,((RESET) | (c<<5)));
	 trigger_LCD_E();
	 delay_us(LCD_INIT_COM_DEL_US);
 }

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		LCD_WriteLineToLCD:	Sendet eine ganze Zeile an das Display inkl. Textausrichtung
					Berechnet die Cursor X-Position anhand der Lnge des bergebenen Strings, setzt den Cursor und
					sendet den String an das Display.
 -------------------------------------------------------------------------------------------------------------------- */
 void LCD_WriteLineToLCD(int line, int textAlign, const char text[])
 {
	int column = 0;
	int len = strlen(text);
	//const char LCD_Text = text;
	if (textAlign == TEXT_LEFT)
	{
		column = 0;
	}
	else if (textAlign == TEXT_CENTER)
	{
		if (len&1)							//ungerade
		{
			column = 16/2-(len+1)/2;
		}
		else								//gerade
		{
			column = 16/2-len/2;
		}
	}
	else if (textAlign == TEXT_RIGHT)
	{
		column = 16 - len;
	}
	else if (textAlign == TEXT_MENU_ITEM)
	{
		column = 1;
	}

	lcd_setCursor(column,line);
	Write_string_lcd(text);
 }

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		LCD_WriteWelcome:	Sendet den Wilkommenstext an das Display.
 -------------------------------------------------------------------------------------------------------------------- */
 void LCD_WriteWelcome(void)
 {
	LCD_WriteLineToLCD(LINE1, TEXT_CENTER, LCD_WELCOME_LINE1);
	LCD_WriteLineToLCD(LINE2, TEXT_CENTER, LCD_WELCOME_LINE2);
	LCD_WriteLineToLCD(LINE3, TEXT_CENTER, LCD_WELCOME_LINE3);
	LCD_WriteLineToLCD(LINE4, TEXT_CENTER, LCD_WELCOME_LINE4);
 }

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		LCD_writeReceivedData:	Sendet die ber RS232 empfangenen Daten an das Display
					Verwaltet die X und Y-Position des Cursors und sendet die empfangenen Daten an das Display.
 -------------------------------------------------------------------------------------------------------------------- */

 //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! TO DO
 void LCD_writeReceivedData (char data)
 {
	if (lcd_rd_cursorX < 16 && lcd_rd_cursorLine != LINE3)
	{
		lcd_rd_cursorLine = LINE2;
	}
	else if (lcd_rd_cursorX == 16 && lcd_rd_cursorLine == LINE2)
	{
		lcd_rd_cursorX = 0;
		lcd_rd_cursorLine = LINE3;
	}
	else if (lcd_rd_cursorX == 16 && lcd_rd_cursorLine == LINE3)
	{
		lcd_rd_cursorLine = LINE2;
		lcd_rd_cursorX = 0;
		lcdClearLine(LINE2);
		lcdClearLine(LINE3);
	}

	if (data != 0x08)
	{
	   	lcd_setCursor(lcd_rd_cursorX, lcd_rd_cursorLine);
	   	Write_char_lcd(data);
	   	lcd_rd_cursorX++;
	}
	else
	{
		if (lcd_rd_cursorX > 0)
		{
			lcd_rd_cursorX--;
			lcd_setCursor(lcd_rd_cursorX, lcd_rd_cursorLine);
			Write_char_lcd(' ');
		}
		else if (lcd_rd_cursorLine == LINE3)
		{
			lcd_rd_cursorLine = LINE2;
			lcd_rd_cursorX = 15;
			lcd_setCursor(lcd_rd_cursorX, lcd_rd_cursorLine);
			Write_char_lcd(' ');
		}
	}
 }

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		checkWelcomeDelay:	Prft ob der Wilkommenstext weiterhin angezeigt werden soll
					Sendet nach Ablauf der Zeit fr den Wilkommenstext die Zeilen fr das Hauptmen.
 -------------------------------------------------------------------------------------------------------------------- */
 void checkWelcomeDelay(void)
 {
	if (lcd_welcomeDelay == LCD_WELCOME_DELAY)
	{
		lcd_selMenu = MENU_LED;
		lcd_subMenu = false;
		lcd_subSubMenu = false;
		lcd_cursorLine = LINE2;
		lcd_scrollLine = 1;
		strncpy(menuLine1, MENU_ITEM_1_LED, sizeof(menuLine1));
		strncpy(menuLine2, MENU_ITEM_2_SUM, sizeof(menuLine2));
		strncpy(menuLine3, MENU_ITEM_3_RS232, sizeof(menuLine3));
		updateMenu();
		lcd_welcomeDelay++;
	}
 }

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		updateMenu:	Aktuallisiert das Hauptmen z.B. nach Tastendruck.
 -------------------------------------------------------------------------------------------------------------------- */
 void updateMenu(void)
 {
	LCD_Clear();
  	LCD_WriteLineToLCD(LINE1, TEXT_CENTER, "***** MENU *****");
  	LCD_WriteLineToLCD(LINE2, TEXT_MENU_ITEM, menuLine1);
  	LCD_WriteLineToLCD(LINE3, TEXT_MENU_ITEM, menuLine2);
  	LCD_WriteLineToLCD(LINE4, TEXT_MENU_ITEM, menuLine3);
  	lcd_setCursor(0, lcd_cursorLine);
  	Write_char_lcd(LCD_MENU_CURSOR);
 }

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		updateSubMenu:	Aktuallisiert dasUntermen z.B. nach Tastendruck.
 -------------------------------------------------------------------------------------------------------------------- */
 void updateSubMenu (void)
 {
	LCD_Clear();
    LCD_WriteLineToLCD(LINE1, TEXT_CENTER, "***** MENU *****");
   	LCD_WriteLineToLCD(LINE2, TEXT_CENTER, selSubMenuText);
	LCD_WriteLineToLCD(LINE3, TEXT_MENU_ITEM, subMenuLine1);
	LCD_WriteLineToLCD(LINE4, TEXT_MENU_ITEM, subMenuLine2);
  	lcd_setCursor(0, lcd_cursorLine);
  	Write_char_lcd(LCD_MENU_CURSOR);
 }

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		ldcClearLine:	Lscht alle Zeichen der angegebenen Zeile.
 -------------------------------------------------------------------------------------------------------------------- */
 void lcdClearLine (int line)
 {
	LCD_WriteLineToLCD(line, TEXT_LEFT, "                ");
 }

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		updateManualTimeDateSetting:	Diese Upadatemethode aktuallisiert die Anzeige im LCD-Display
													whrend der manuellen Einstellung der Uhrzeit.
 -------------------------------------------------------------------------------------------------------------------- */
 void updateManualTimeDateSetting (uint8_t timeOrDate)
 {
	char hour_str[2], min_str[2];
	char year_str[4], month_str[2], day_str[2], week_str[2];

	lcdClearLine(LINE1);
	lcdClearLine(LINE3);

	if (timeOrDate == TIME)
	{
		memset(timeSetting_time_str, 0, strlen(timeSetting_time_str));
		LCD_WriteLineToLCD(LINE1, TEXT_CENTER, "B3 = confirm");
		sprintf(hour_str, "%ld", manualTime_hour);
		sprintf(min_str, "%ld", manualTime_min);
		strcat(timeSetting_time_str, "Time: ");
		if(manualTime_hour < 10){strcat(timeSetting_time_str, "0");}
		strcat(timeSetting_time_str, hour_str);
		strcat(timeSetting_time_str, ":");
		if(manualTime_min < 10){strcat(timeSetting_time_str, "0");}
		strcat(timeSetting_time_str, min_str);
		strcat(timeSetting_time_str, " Uhr");
		LCD_WriteLineToLCD(LINE3, TEXT_LEFT, timeSetting_time_str);
	}
	else if (timeOrDate == DATE)
	{
		memset(timeSetting_date_str, 0, strlen(timeSetting_date_str));
		LCD_WriteLineToLCD(LINE1, TEXT_CENTER, "B3 = confirm");
		sprintf(day_str, "%ld", manualDate_day);
		sprintf(month_str, "%ld", manualDate_month);
		sprintf(year_str, "%ld", manualDate_year);
		strcat(timeSetting_date_str, "Date: ");
		if(manualDate_day < 10){strcat(timeSetting_date_str, "0");}
		strcat(timeSetting_date_str, day_str);
		strcat(timeSetting_date_str, ".");
		if(manualDate_month < 10){strcat(timeSetting_date_str, "0");}
		strcat(timeSetting_date_str, month_str);
		strcat(timeSetting_date_str, ".");
		strcat(timeSetting_date_str, year_str);
		LCD_WriteLineToLCD(LINE3, TEXT_LEFT, timeSetting_date_str);
	}
	else if (timeOrDate == WEEK)
	{
		memset(timeSetting_week_str, 0, strlen(timeSetting_week_str));
		LCD_WriteLineToLCD(LINE1, TEXT_CENTER, "B3 = confirm");
		sprintf(week_str, "%ld", manualDate_week);
		strcat(timeSetting_week_str, "Weekday: ");
		switch (manualDate_week)
		{
			case 1:
			strcat(timeSetting_week_str, "Mo.");	break;
			case 2:
			strcat(timeSetting_week_str, "Tu.");	break;
			case 3:
			strcat(timeSetting_week_str, "We.");	break;
			case 4:
			strcat(timeSetting_week_str, "Th.");	break;
			case 5:
			strcat(timeSetting_week_str, "Fr.");	break;
			case 6:
			strcat(timeSetting_week_str, "Sa.");	break;
			case 7:
			strcat(timeSetting_week_str, "Su.");	break;
		}
		LCD_WriteLineToLCD(LINE3, TEXT_LEFT, timeSetting_week_str);
	}
 }