/*	
 *	Pollin Electronic GmbH
 *	Max-Pollin-Strae 1			
 *	85104 Pfrring 
 *	http://www.pollin.de
 *
 *	Project:	RAPITUS SAMxS Evaluationboard 
 *  Filename:	lcd.h
 *
 *  Author:		Leonhard Hesse
 *
 *  Copyright (c) 2016 Pollin Electronic GmbH. All rights reserved.
 *
 *  Bitte beachten Sie, dass dieses Programm nur uneingeschrnkt fr nicht kommerzielle Zwecke genutzt 
 *  werden darf. Ich bernehme keine Garantie fr die einwandfreie Funktion dieses Programmes und ber- 
 *  nehme dadurch auch keine Haftung fr Folgeschden die Aufgrund der Benutzung dieser Daten entstehen. 
 *  Dieses gilt auch dann, wenn ich ber einen solchen Schaden unterrichtet worden bin. 
 */

#include "compiler.h"
#include "stdbool.h"

#ifndef LCD_H_
#define LCD_H_

#define LCD_MASK_ALL			0b00000000000000000001111111101100
#define LCD_MASK_DB				0b00000000000000000001111111100000
#define LCD_MASK_RS				0b00000000000000000000000000000100
#define LCD_MASK_E				0b00000000000000000000000000001000
#define LCD_E					PIO_PC3_IDX

#define RESET					0x00		//BIN:	0000 0000
#define LCD_INIT_ATT			0x30		//BIN:	0011 0000
#define LCD_INIT_FUNC			0x38		//BIN:	0011 1000
#define LCD_INIT_OFF			0x08		//BIN:	0000 1000
#define LCD_INIT_ON				0x0C		//BIN:	0000 1100
#define LCD_INIT_MODE			0x06		//BIN:	0000 0110
#define LCD_INIT_CLEAR			0x01		//BIN:	0000 0001
#define LCD_RS_DATA				0x01		//BIN:	0000 0001

#define LCD_BOOT_DEL_MS			20
#define LCD_INIT_ATT_DEL_MS		5
#define LCD_INIT_ATT_DEL_US		100
#define LCD_INIT_COM_DEL_US		40
#define LCD_INIT_CLEAR_DEL_MS	2
#define LCD_TRIG_E_DEL_US		1

#define LCD_SET_DDADR           0x80
#define LCD_DDADR_LINE1         0x00
#define LCD_DDADR_LINE2         0x40
#define LCD_DDADR_LINE3         0x10
#define LCD_DDADR_LINE4         0x50

#define TEXT_LEFT				0
#define TEXT_CENTER				1
#define TEXT_RIGHT				2
#define TEXT_MENU_ITEM			3

#define LINE1					1
#define LINE2					2
#define LINE3					3
#define LINE4					4

#define LCD_WELCOME_LINE1		"PollinElectronic"
#define LCD_WELCOME_LINE2		"Welcome to the"
#define LCD_WELCOME_LINE3		"ARM SAM-XS"
#define LCD_WELCOME_LINE4		"Evaluationboard!"
#define LCD_WELCOME_DELAY		300

#define LCD_MENU_CURSOR			0x7E
#define LCD_MENU_CURSOR_POS1	3
#define LCD_MENU_CURSOR_POS2	4

#define LCD_TIME_CURSOR_R		0x7E
#define LCD_TIME_CURSOR_L		0x7F

#define MENU_LED				1
#define	MENU_SUMMER				2
#define MENU_RS232				3
#define MENU_TIME				4
#define MENU_RFM				5

#define MENU_ITEM_1_LED			"LEDs"
#define MENU_ITEM_2_SUM			"Soundcheck"
#define MENU_ITEM_3_RS232		"RS232 RX/TX"
#define MENU_ITEM_4_TIME		"Time Settings"
#define MENU_ITEM_5_RFM			"RFM75"

#define MENU_SUBITEM_BACK		"..exit"

#define MENU_SUBITEM_LED_1		"Toggle LED 1"
#define MENU_SUBITEM_LED_2		"Toggle LED 2"
#define MENU_SUBITEM_LED_3		"Toggle LED 1+2"

#define MENU_SUBITEM_SUMMER		"Buzzer on/off"

#define MENU_SUBITEM_RS232_1	"RS232 J2"
#define MENU_SUBITEM_RS232_2	"RS232 J3"
#define MENU_SUBSUBITEM_RS232_1	"Transmit data"
#define MENU_SUBSUBITEM_RS232_2	"Receive data"
#define TRANSMIT_DATA			"Hello! My name is Sam. "

#define MENU_SUBITEM_TIME_1		"Set via DCF"
#define MENU_SUBITEM_TIME_2		"Set manual"
#define MENU_SUBITEM_TIME_3		"Show Time&Date"

#define MENU_SUBITEM_RFM_RX		"RFM RX data:"
#define MENU_SUBITEM_RFM_TX		"Transmit data"

#define TIME 1
#define DATE 2
#define WEEK 3

volatile uint16_t lcd_welcomeDelay;

uint8_t lcd_selMenu;
uint8_t lcd_cursorLine;
uint8_t lcd_selSubMenu;
uint8_t lcd_selSubSubMenu;
uint8_t lcd_scrollLine;
uint8_t lcd_rd_cursorX;
uint8_t lcd_rd_cursorLine;

uint32_t manualTime_hour;
uint32_t manualTime_min;
uint32_t manualDate_day;
uint32_t manualDate_month;
uint32_t manualDate_year;
uint32_t manualDate_week;
uint32_t selTimeSetting;

bool lcd_subMenu;
bool lcd_subSubMenu;
bool lcd_lastMenu;
bool lcd_showReceivedData;
bool lcd_showReceivedDataRFM;
bool lcd_showTime;

bool timeSetManualDone;
bool summerOn;
bool summerStart;

char menuLine1[16];
char menuLine2[16];
char menuLine3[16];
char selSubMenuText[16];
char subMenuLine1[16];
char subMenuLine2[16];

void LCD_Init(void);
void LCD_Clear(void);
void LCD_WriteLineToLCD(int line, int textAlign, const char text[]);
void trigger_LCD_E(void);
void Write_string_lcd(const char * s);
void Write_char_lcd(unsigned char c);
void lcd_setCursor( uint8_t x, uint8_t y );
void LCD_WriteWelcome(void);
void checkWelcomeDelay(void);
void updateMenu(void);
void manageMenu(int button);
void updateSubMenu (void);
void LCD_writeReceivedData (char data);
void lcdClearLine (int line);
void updateManualTimeDateSetting (uint8_t timeOrDate);
void show_LCD_MainMenu (void);

#endif /* LCD_H_ */