/*	
 *	Pollin Electronic GmbH
 *	Max-Pollin-Strae 1			
 *	85104 Pfrring 
 *	http://www.pollin.de
 *
 *	Project:	RAPITUS SAMxS Evaluationboard 
 *  Filename:	rfm.c
 *
 *  Author:		Leonhard Hesse
 *
 *  Copyright (c) 2016 Pollin Electronic GmbH. All rights reserved.
 *
 *  Bitte beachten Sie, dass dieses Programm nur uneingeschrnkt fr nicht kommerzielle Zwecke genutzt 
 *  werden darf. Ich bernehme keine Garantie fr die einwandfreie Funktion dieses Programmes und ber- 
 *  nehme dadurch auch keine Haftung fr Folgeschden die Aufgrund der Benutzung dieser Daten entstehen. 
 *  Dieses gilt auch dann, wenn ich ber einen solchen Schaden unterrichtet worden bin. 
 */

#include <asf.h>
#include "rfm.h"
#include "rfm75.h"
#include "mySpi.h"
#include "uart.h"
#include "sound.h"
#include "string.h"
#include "uart_menu.h"
#include "rfm.h"
#include "lcd.h"

 #if defined RFM_RS232_J3 && defined RFM_TRANSMITTER
 bool rfm_J3_def = true;
 #else
 bool rfm_J3_def = false;
 #endif

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		rfrm_init:				Initialisierung des RFM-Moduls und aktivieren des Interrupts fr PIOA
 -------------------------------------------------------------------------------------------------------------------- */
void rfm_init(void)
{
	/* Set RF channel */
	rfm_writeRegister(RF_CH, MODULE_CHANNEL);
	/* Set length of payload */
	rfm_writeRegister(RX_PW_P0, MODULE_PAYLOAD);
	/* Enable clock and interrupt */
	pmc_enable_periph_clk(RFM_IRQ_ID);
	NVIC_EnableIRQ(PIOA_IRQn);
}

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		rfrm_select:			Auswhlen des RFM-Moduls fr SPI. (Chip Select -> GND)
 -------------------------------------------------------------------------------------------------------------------- */
void rfm_select (void)
{
	gpio_set_pin_low(SPI_NPCS0_GPIO);
}

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		rfrm_deselect:			Abwhlen des RFM-modul fr SPI. (Chip Select -> 3,3V)
 -------------------------------------------------------------------------------------------------------------------- */
void rfm_deselect (void)
{
	gpio_set_pin_high(SPI_NPCS0_GPIO);
	delay_us(5);
}

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		rfrm_writeRegister:		Schreibt ein Byte in das angegebene Register des RFM-Moduls.
	Parameter:		Input:					reg		Das Register, aus das zugegriffen werden soll.
											value	Byte das in das Register geschrieben werden soll.
 -------------------------------------------------------------------------------------------------------------------- */
void rfm_writeRegister (uint8_t reg, uint8_t value)
{
	rfm_select();
	mySpi_rw(SPI, W_REGISTER | (REGISTER_MASK & reg), SPI_PCS_RFM, SPI_LAST_BYTE_FALSE);
	mySpi_rw(SPI, value, SPI_PCS_RFM, SPI_LAST_BYTE_TRUE);
	rfm_deselect();
}

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		rfrm_readRegister:		Liest das angegebene Register des RFM-Moduls.
	Parameter:		Input:					reg		Das Register, aus das zugegriffen werden soll.
											value	Byte das in das Register geschrieben werden soll.
					Return:	reg_value		Inhalt des Status-Registers.
 -------------------------------------------------------------------------------------------------------------------- */
uint8_t rfm_readRegister (uint8_t reg)
{
	uint8_t reg_value;
	rfm_select();
	mySpi_rw(SPI, R_REGISTER | (REGISTER_MASK & reg), SPI_PCS_RFM, SPI_LAST_BYTE_FALSE);
	reg_value = mySpi_rw(SPI, 0x00, SPI_PCS_RFM, SPI_LAST_BYTE_TRUE);
	rfm_deselect();
	return reg_value;
}

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		rfrm_writeRegisterArray:	Schreibt ein Array an Daten in das angegebene Register des RFM-Moduls.
	Parameter:		Input:						reg			Das Register, aus das zugegriffen werden soll.
												*array		Pointer auf das zu sendende Datenpaket (Array).
												len			Lnge des Arrays.
 -------------------------------------------------------------------------------------------------------------------- */
void rfm_writeRegisterArray (uint8_t reg, uint8_t * array, uint8_t len)
{
	rfm_select();
	mySpi_rw(SPI, W_REGISTER | (REGISTER_MASK & reg), SPI_PCS_RFM, SPI_LAST_BYTE_FALSE);
	for (int i = 0; i < len-1; i++)
	{
		mySpi_rw(SPI, array[i], SPI_PCS_RFM, SPI_LAST_BYTE_FALSE);
	}
	mySpi_rw(SPI, array[len-1], SPI_PCS_RFM, SPI_LAST_BYTE_TRUE);
	rfm_deselect();
}

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		rfrm_writePayload:	Schreibt die zu sendenden Daten in die Register des RFM-Moduls
	Parameter:		Input:	*array		Pointer auf das zu sendende Datenpaket (Array).
							len			Lnge des Arrays.
 -------------------------------------------------------------------------------------------------------------------- */
void rfm_writePayload (uint8_t * array, uint8_t len)
{
	rfm_select();
	mySpi_rw(SPI, W_TX_PAYLOAD, SPI_PCS_RFM, SPI_LAST_BYTE_FALSE);
	for (int i = 0; i < len-1; i++)
	{
		mySpi_rw(SPI, array[i], SPI_PCS_RFM, SPI_LAST_BYTE_FALSE);
	}
	mySpi_rw(SPI, array[len-1], SPI_PCS_RFM, SPI_LAST_BYTE_TRUE);
	rfm_deselect();
}

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		rfm_readPayload:	Empfnger - auslesen der Empfangenen Daten
	Parameter:		Input:	*array		Pointer auf ein Array zum abspeichern der empfangenen Daten.
							len			Lnge des Arrays.
 -------------------------------------------------------------------------------------------------------------------- */
void rfm_readPayload (uint8_t * array, uint8_t len)
{
	rfm_select();
	mySpi_rw(SPI, R_RX_PAYLOAD, SPI_PCS_RFM, SPI_LAST_BYTE_FALSE);
	for (int i = 0; i < len-1; i++)
	{
		array[i] = mySpi_rw(SPI, 0x00, SPI_PCS_RFM, SPI_LAST_BYTE_FALSE);
	}
	array[len-1] = mySpi_rw(SPI,0x00, SPI_PCS_RFM, SPI_LAST_BYTE_TRUE);
	rfm_deselect();
}

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		rfm_getStatus:		Abfragen des Status-Registers.
										Fragt das Status-Register ab und gibt den Inhalt zurck.
	Parameter:		Return:	reg_value	Inhalt des Status-Registers.
 -------------------------------------------------------------------------------------------------------------------- */
uint8_t rfm_getStatus (void) 
{
	uint8_t reg_value;
	rfm_select();
	reg_value = mySpi_rw(SPI, NOP, SPI_PCS_RFM, SPI_LAST_BYTE_TRUE);
	rfm_deselect();
	return reg_value;
}

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		rfm_sendArray:	Sendet ein Array Daten an den zuvor festgelegten Empfnger.
	Parameter:		Input:	*array	Pointer auf die zu sendenden Daten.
							len		Die Lnge des Arrays.
 -------------------------------------------------------------------------------------------------------------------- */
void rfm_sendArray (uint8_t * array, uint8_t len)
{
		while (rfm_PTX)
		
		gpio_set_pin_low(RFM_CE_GPIO);
		rfm_PTX = 1;
		TX_POWERUP;

		rfm_select();
		mySpi_rw(SPI, FLUSH_TX, SPI_PCS_RFM, SPI_LAST_BYTE_TRUE);
		rfm_deselect();

		rfm_writePayload(array, len);

		gpio_set_pin_high(RFM_CE_GPIO);
		delay_us(15);
		gpio_set_pin_low(RFM_CE_GPIO);
}

 /* --------------------------------------------------------------------------------------------------------------------
	Methode:		rfm_configTX:	Konfiguriert das RFM-Modul as Sender
									Nach der Konfiguration der einzelnen Register wird fr jeden der sechs Kanle
									die standard Adresse gesetzt. In dieser Demo wird nur Pipe 0 verwendet.
	Parameter:		Input:	tx_nr	Verwendeter Kanal (Pipe)
 -------------------------------------------------------------------------------------------------------------------- */
void rfm_configTX (uint8_t tx_nr)
{
	uint8_t tx_addr[5];

	/* Set RF channel */
	rfm_writeRegister(RF_CH, MODULE_CHANNEL);
	/* Set data speed & output power */
	rfm_writeRegister(RF_SETUP, MODULE_RF_SETUP);
	/* Config the CONFIG register */
	rfm_writeRegister(CONFIG, MODULE_CONFIG);
	/* Config retransmission delay & retransmission count */
	rfm_writeRegister(SETUP_RETR, (SETUP_RETR_ARD_750 | SETUP_RETR_ARC_15));

	switch(tx_nr)
	{
		case 0: //setup TX address as default RX address for pipe 0 (E7:E7:E7:E7:E7)
		tx_addr[0] = tx_addr[1] = tx_addr[2] = tx_addr[3] = tx_addr[4] = RX_ADDR_P0_B0_DEFAULT_VAL;
		rfm_writeRegisterArray(TX_ADDR, tx_addr, 5);
		gpio_set_pin_low(RFM_CE_GPIO);
		rfm_writeRegisterArray(RX_ADDR_P0, tx_addr, 5);
		gpio_set_pin_high(RFM_CE_GPIO);
		break;
		case 1: //setup TX address as default RX address for pipe 1 (C2:C2:C2:C2:C2)
		tx_addr[0] = tx_addr[1] = tx_addr[2] = tx_addr[3] = tx_addr[4] = RX_ADDR_P1_B0_DEFAULT_VAL;
		rfm_writeRegisterArray(TX_ADDR, tx_addr, 5);
		gpio_set_pin_low(RFM_CE_GPIO);
		rfm_writeRegisterArray(RX_ADDR_P0, tx_addr, 5);
		gpio_set_pin_high(RFM_CE_GPIO);
		break;
		case 2: //setup TX address as default RX address for pipe 2 (C2:C2:C2:C2:C3)
		tx_addr[1] = tx_addr[2] = tx_addr[3] = tx_addr[4] = RX_ADDR_P1_B0_DEFAULT_VAL;
		tx_addr[0] = RX_ADDR_P2_DEFAULT_VAL;
		rfm_writeRegisterArray(TX_ADDR, tx_addr, 5);
		gpio_set_pin_low(RFM_CE_GPIO);
		rfm_writeRegisterArray(RX_ADDR_P0, tx_addr, 5);
		gpio_set_pin_high(RFM_CE_GPIO);
		break;
		case 3: //setup TX address as default RX address for pipe 3 (C2:C2:C2:C2:C4)
		tx_addr[1] = tx_addr[2] = tx_addr[3] = tx_addr[4] = RX_ADDR_P1_B0_DEFAULT_VAL;
		tx_addr[0] = RX_ADDR_P3_DEFAULT_VAL;
		rfm_writeRegisterArray(TX_ADDR, tx_addr, 5);
		gpio_set_pin_low(RFM_CE_GPIO);
		rfm_writeRegisterArray(RX_ADDR_P0, tx_addr, 5);
		gpio_set_pin_high(RFM_CE_GPIO);
		break;
		case 4: //setup TX address as default RX address for pipe 4 (C2:C2:C2:C2:C5)
		tx_addr[1] = tx_addr[2] = tx_addr[3] = tx_addr[4] = RX_ADDR_P1_B0_DEFAULT_VAL;
		tx_addr[0] = RX_ADDR_P4_DEFAULT_VAL;
		rfm_writeRegisterArray(TX_ADDR, tx_addr, 5);
		gpio_set_pin_low(RFM_CE_GPIO);
		rfm_writeRegisterArray(RX_ADDR_P0, tx_addr, 5);
		gpio_set_pin_high(RFM_CE_GPIO);
		break;
		case 5: //setup TX address as default RX address for pipe 5 (C2:C2:C2:C2:C6)
		tx_addr[1] = tx_addr[2] = tx_addr[3] = tx_addr[4] = RX_ADDR_P1_B0_DEFAULT_VAL;
		tx_addr[0] = RX_ADDR_P5_DEFAULT_VAL;
		rfm_writeRegisterArray(TX_ADDR, tx_addr, 5);
		gpio_set_pin_low(RFM_CE_GPIO);
		rfm_writeRegisterArray(RX_ADDR_P0, tx_addr, 5);
		gpio_set_pin_high(RFM_CE_GPIO);
		break;
	}

	rfm_PTX = 0;
	TX_POWERUP;
}

void rfm_configRX (void)
{
	/* Set RF channel */
	rfm_writeRegister(RF_CH, MODULE_CHANNEL);
	/* Set data speed & output power */
	rfm_writeRegister(RF_SETUP, MODULE_RF_SETUP);
	/* Config the CONFIG register */
	rfm_writeRegister(CONFIG, MODULE_CONFIG_RX);

	rfm_PTX = 0;
	RX_POWERUP;
	gpio_set_pin_high(RFM_CE_GPIO);
}

/* --------------------------------------------------------------------------------------------------------------------
	PIOA interrupt Handler			rfm_handler: Interpretation des Interrupts vom RFM Modul
									Wird ein Interrupt vom RFM Modul erkannt, wird zunchst das Status Register
									ausgelesen, um den Auslser des Interrupts zu ermitteln. 
									Modul als Empfnger:
										- RFM_RS232_J3 ist definiert: empfangene Daten werden an J3 ausgegeben.
										- Im Men wird RFM75 gewhlt: empfangene Daten werden im Men angezeigt.
									Modul als Sender:
										- RFM_RS232_J3 ist definiert: an J3 empfangene Daten werden gesendet.
										- Im Men wird RFM75 gewhlt: im Men eingegebene Daten werden gesendet.
 -------------------------------------------------------------------------------------------------------------------- */

void rfm_handler(uint32_t id, uint32_t index)
{
	if ((id == RFM_IRQ_ID) && (index == RFM_IRQ_MASK))
	{
		uint8_t status;
		status = rfm_getStatus();							// read status register

		/* Interrupt routine for transmitter **************************************************************************/
			
		if (status & (1<<TX_DS))							// package has been sent
		{
			rfm_writeRegister (STATUS, (1<<TX_DS));			// clear interrupt bit
			rfm_PTX = 0;
		}
		
		if (status & (1<<MAX_RT))							// package has not been sent
		{
			rfm_writeRegister (STATUS, (1<<MAX_RT));		// clear interrupt bit
			if (rfm_sendCounter < 100)
			{
				gpio_set_pin_high(RFM_CE_GPIO);				// send again
				delay_us(10);
				gpio_set_pin_low(RFM_CE_GPIO);
				rfm_sendCounter++;
			}
			else
			{
				rfm_select()  ;													// pull down chip select
				mySpi_rw(SPI, FLUSH_TX, SPI_PCS_RFM, SPI_LAST_BYTE_TRUE);		// flush TX-FIFO
				rfm_deselect();													// pull up chip select
				uart_write_string(UART1, "Receiver not available. :(\r");
				uart_write_string(UART1, "Transmission canceled and FIFO cleared. Please try again.\r");
				rfm_secondCounter = 0;
				rfm_PTX = 0;
			}		
		}
		
		if (status & (1<<TX_FULL))							//TX_FIFO Full <-- this is no interrupt
		{
			rfm_select()  ;													// pull down chip select
			mySpi_rw(SPI, FLUSH_TX, SPI_PCS_RFM, SPI_LAST_BYTE_TRUE);		// flush TX-FIFO
			rfm_deselect();													// pull up chip select
		}

		/* Interrupt routine for receiver *****************************************************************************/

		if (status & (1<<RX_DR))							// received data ready
		{
			uint8_t data[MODULE_PAYLOAD];
			rfm_readPayload(data, MODULE_PAYLOAD);			// store received data
			rfm_writeRegister (STATUS, (1<<RX_DR));			// clear interrupt bit

			char string[MODULE_PAYLOAD];					// prepare data to send as string via uart
			memset(string, 0, strlen(string));
			memcpy(string, data, MODULE_PAYLOAD);

			/* check user settings if via RFM received data should be sent on J3  */
			if (rfm_J3_def == true)
			{
				uart_write_string(UART1, string);
			}
			beep();											// visualize received data

			if (uart_receiveDataRFM == true)
			{
				uart_write_string(UART0, string);
			}

			if (lcd_showReceivedDataRFM == true)
			{
				for (int i = 0; i < MODULE_PAYLOAD; i++)
				{
					if (string[i] != 0)
					{
						if (string[i] == 27)		//ESC
						{
							lcdClearLine(LINE2);
							lcdClearLine(LINE3);
							lcd_rd_cursorLine = LINE2;
							lcd_rd_cursorX = 0;
						}
						else
						{
							LCD_writeReceivedData(string[i]);
						}					
					}
				}
			}
		}	
	}
}