/*	
 *	Pollin Electronic GmbH
 *	Max-Pollin-Strae 1			
 *	85104 Pfrring 
 *	http://www.pollin.de
 *
 *	Project:	RAPITUS SAMxS Evaluationboard 
 *  Filename:	rtc.c 
 *
 *  Author:		Leonhard Hesse
 *
 *  Copyright (c) 2016 Pollin Electronic GmbH. All rights reserved.
 *
 *  Bitte beachten Sie, dass dieses Programm nur uneingeschrnkt fr nicht kommerzielle Zwecke genutzt 
 *  werden darf. Ich bernehme keine Garantie fr die einwandfreie Funktion dieses Programmes und ber- 
 *  nehme dadurch auch keine Haftung fr Folgeschden die Aufgrund der Benutzung dieser Daten entstehen. 
 *  Dieses gilt auch dann, wenn ich ber einen solchen Schaden unterrichtet worden bin. 
 */

 #include "asf.h"
 #include "dcf.h"
 #include "lcd.h"
 #include "rtc.h"
 #include "uart_menu.h"
 #include "string.h"

 uint32_t dcf_startup = 0;

 void rtc_setup(void)
 {
	 pmc_switch_sclk_to_32kxtal(PMC_OSC_XTAL);
	 while (!pmc_osc_is_ready_32kxtal());
	 rtc_set_hour_mode(RTC, 0);
	 rtc_enable_interrupt(RTC, RTC_IER_SECEN);
	 NVIC_EnableIRQ(RTC_IRQn);
 }

 void rtc_update(void)
 {
	rtc_get_time(RTC, &hour, &minute, &second);
	rtc_get_date(RTC, &year, &month, &day, &week);
	memset(time_str, 0, strlen(time_str));
	memset(date_str, 0, strlen(date_str));
	sprintf(hour_str, "%ld", hour);
	sprintf(min_srt, "%ld", minute);
	sprintf(sek_str, "%ld", second);
	sprintf(year_str, "%ld", year);
	sprintf(month_str, "%ld", month);
	sprintf(day_str, "%ld", day);

	if (hour<10){strcat(time_str, "0");}
	strcat(time_str, hour_str);
	strcat(time_str, ":");
	if (minute<10){strcat(time_str, "0");}
	strcat(time_str, min_srt);
	strcat(time_str, ":");
	if (second<10){strcat(time_str, "0");}
	strcat(time_str, sek_str);

	switch (week)
	{
		case 1:
		strcat(date_str, "Mo. ");	break;
		case 2:
		strcat(date_str, "Tu. ");	break;
		case 3:
		strcat(date_str, "We. ");	break;
		case 4:
		strcat(date_str, "Th. ");	break;
		case 5:
		strcat(date_str, "Fr. ");	break;
		case 6:
		strcat(date_str, "Sa. ");	break;
		case 7:
		strcat(date_str, "Su. ");	break;
	}

	if (day<10){strcat(date_str, "0");}
	strcat(date_str, day_str);
	strcat(date_str, ".");
	if (month<10){strcat(date_str, "0");}
	strcat(date_str, month_str);
	strcat(date_str, ".");
	strcat(date_str, year_str);
 }

 void RTC_Handler(void)
 {
	 rtc_clear_status(RTC, RTC_SR_SEC);

	 if ((dcf_validTimeReceived == true && dcf_validDateReceived == true) || timeSetManualDone == true || lcd_showTime == true)
	 {
		 rtc_update();
		 LCD_Clear();
		 if (lcd_showTime == true)
		 {
			 LCD_WriteLineToLCD(LINE1, TEXT_CENTER, "Time & Date");
			 LCD_WriteLineToLCD(LINE4, TEXT_MENU_ITEM, MENU_SUBITEM_BACK);
		 }
		 else
		 {
			 LCD_WriteLineToLCD(LINE1, TEXT_CENTER, "(: SUCCESS! :)");
			 LCD_WriteLineToLCD(LINE4, TEXT_MENU_ITEM, "finish..");
			 dcf_completed = true;
		 }
		 LCD_WriteLineToLCD(LINE2, TEXT_CENTER, time_str);
		 LCD_WriteLineToLCD(LINE3, TEXT_CENTER, date_str);
		 lcd_setCursor(0, LINE4);
		 Write_char_lcd(LCD_MENU_CURSOR);

		 uart_showReceivedData();
	 }

	 if (uart_updateTimeAndDate == true)
	 {
		rtc_update();
		uart_updateTimeDate();
	 }
 }