/*	
 *	Pollin Electronic GmbH
 *	Max-Pollin-Strae 1			
 *	85104 Pfrring 
 *	http://www.pollin.de
 *
 *	Project:	RAPITUS SAMxS Evaluationboard 
 *  Filename:	timer.c 
 *
 *  Author:		Leonhard Hesse
 *
 *  Copyright (c) 2016 Pollin Electronic GmbH. All rights reserved.
 *
 *  Bitte beachten Sie, dass dieses Programm nur uneingeschrnkt fr nicht kommerzielle Zwecke genutzt 
 *  werden darf. Ich bernehme keine Garantie fr die einwandfreie Funktion dieses Programmes und ber- 
 *  nehme dadurch auch keine Haftung fr Folgeschden die Aufgrund der Benutzung dieser Daten entstehen. 
 *  Dieses gilt auch dann, wenn ich ber einen solchen Schaden unterrichtet worden bin. 
 */

  #include "asf.h"
  #include "timer.h"
  #include "lcd.h"
  #include "sound.h"
  #include "mySpi.h"
  #include "main.h"
  #include "dcf.h"
  #include "string.h"
  #include "uart.h"
  #include "uart_menu.h"

 // Configuration function for the Timer.
 void tc_config_TC0(uint32_t freq_desired)
 {
	 // INPUTS:
	 // freq_desired	The desired rate at which to call the ISR, Hz.
	 
	 uint32_t ul_div;
	 uint32_t ul_tcclks;
	 uint32_t ul_sysclk = sysclk_get_cpu_hz();
	 uint32_t counts;
	 
	 // Configure PMC
	 pmc_enable_periph_clk(ID_TC0);

	 // Configure TC for a 100Hz frequency and trigger on RC compare.
	 tc_find_mck_divisor(
	 (uint32_t)freq_desired,	// The desired frequency as a uint32.
	 ul_sysclk,					// Master clock freq in Hz.
	 &ul_div,					// Pointer to register where divisor will be stored.
	 &ul_tcclks,				// Pointer to reg where clock selection number is stored.
	 ul_sysclk);				// Board clock freq in Hz.
	 tc_init(TC0, CHANNEL_TIMER_0, ul_tcclks | TC_CMR_CPCTRG);
	 
	 // Find the best estimate of counts, then write it to TC register C.
	 counts = (ul_sysclk/ul_div)/freq_desired;
	 tc_write_rc(TC0, CHANNEL_TIMER_0, counts);

	 // Enable interrupts for this TC, and start the TC.
	 tc_enable_interrupt(TC0, CHANNEL_TIMER_0, TC_IER_CPCS);				// Enable interrupt.
	 tc_start(TC0,CHANNEL_TIMER_0);											// Start the TC.

	 NVIC_EnableIRQ(TC0_IRQn);
 }

 // Timer Interrupt service routine - every 10ms - used for ADC and counter for the welcome page on the lcd
 void TC0_Handler(void)
 {
	 uint32_t dummy;
	 dummy = tc_get_status(TC0,CHANNEL_TIMER_0);
	 UNUSED(dummy);

	 adc_start(ADC);

	 if (lcd_welcomeDelay < LCD_WELCOME_DELAY)
	 {
		lcd_welcomeDelay++;
	 }
	 else
	 {
		checkWelcomeDelay();
	 }	 
	 
 }

  // Configuration function for the Timer.
  void tc_config_TC1(uint32_t freq_desired)
  {
	  // INPUTS:
	  // freq_desired	The desired rate at which to call the ISR, Hz.
	  
	  uint32_t ul_div;
	  uint32_t ul_tcclks = 1;
	  uint32_t ul_sysclk = sysclk_get_cpu_hz();
	  uint32_t counts;
	  
	  // Configure PMC
	  pmc_enable_periph_clk(ID_TC1);

	  // Configure TC for a 10KHz frequency and trigger on RC compare.
	  tc_find_mck_divisor(
	  (uint32_t)freq_desired,	// The desired frequency as a uint32.
	  ul_sysclk,				// Master clock freq in Hz.
	  &ul_div,					// Pointer to register where divisor will be stored.
	  &ul_tcclks,				// Pointer to reg where clock selection number is stored.
	  ul_sysclk);				// Board clock freq in Hz.
	  tc_init(TC0, CHANNEL_TIMER_1, ul_tcclks | TC_CMR_CPCTRG);
	  
	  // Find the best estimate of counts, then write it to TC register C.
	  counts = (ul_sysclk/ul_div)/freq_desired;
	  tc_write_rc(TC0, CHANNEL_TIMER_1, counts);

	  // Enable interrupts for this TC, and start the TC.
	  tc_enable_interrupt(TC0, CHANNEL_TIMER_1, TC_IER_CPCS);				// Enable interrupt.
	  tc_start(TC0,CHANNEL_TIMER_1);										// Start the TC.

	  NVIC_EnableIRQ(TC1_IRQn);
  }

  // Timer Interrupt service routine - every 100us - used for buzzer
  void TC1_Handler(void)
  {
	  uint32_t dummy;
	  dummy = tc_get_status(TC0,CHANNEL_TIMER_1);
	  UNUSED(dummy);

	  playSound();
  }

   // Configuration function for the Timer.
   void tc_config_TC2(uint32_t freq_desired)
   {
	   // INPUTS:
	   // freq_desired	The desired rate at which to call the ISR, Hz.
	   
	   uint32_t ul_div;
	   uint32_t ul_tcclks;
	   uint32_t ul_sysclk = sysclk_get_cpu_hz();
	   uint32_t counts;
	   
	   // Configure PMC
	   pmc_enable_periph_clk(ID_TC2);

	   // Configure TC for a 100Hz frequency and trigger on RC compare.
	   tc_find_mck_divisor(
	   (uint32_t)freq_desired,	// The desired frequency as a uint32.
	   ul_sysclk,					// Master clock freq in Hz.
	   &ul_div,					// Pointer to register where divisor will be stored.
	   &ul_tcclks,				// Pointer to reg where clock selection number is stored.
	   ul_sysclk);				// Board clock freq in Hz.
	   tc_init(TC0, CHANNEL_TIMER_2, ul_tcclks | TC_CMR_CPCTRG);
	   
	   // Find the best estimate of counts, then write it to TC register C.
	   counts = (ul_sysclk/ul_div)/freq_desired;
	   tc_write_rc(TC0, CHANNEL_TIMER_2, counts);

	   // Enable interrupts for this TC, and start the TC.
	   tc_enable_interrupt(TC0, CHANNEL_TIMER_2, TC_IER_CPCS);				// Enable interrupt.

	   tc_start(TC0,CHANNEL_TIMER_2);	

	   NVIC_EnableIRQ(TC2_IRQn);
   }

   // Timer Interrupt service routine - every 1ms - used for DCF
   void TC2_Handler(void)
   {
	   uint32_t dummy;
	   dummy = tc_get_status(TC0,CHANNEL_TIMER_2);
	   UNUSED(dummy);

	   if (dcf_completed == false)		// If already received time and date do nothing here
	   {
		   dcf_detectBitCounter++;
	   
		   if (dcf_errorDetection > 0)
		   {
				dcf_errorDetection++;   
				if (dcf_errorDetection > 950)		// valid data only possible when no signal for mind 0,95s
				{
					dcf_invalidSignal = false;
				}	   
				else
				{
					dcf_invalidSignal = true;
				}
			}

			if (dcf_startup < DCF_STARTUPTIME)		// DCF-Modul starts on falling edge
			{
				gpio_set_pin_low(DCF_PON_GPIO);
				dcf_startup++;
			}
			else if (dcf_startup == DCF_STARTUPTIME)	// wait until the DCF-Modul has started then start signal test
			{				
				dcf_startup++;
				LCD_Clear();
				LCD_WriteLineToLCD(LINE1, TEXT_LEFT, "wait for signal");
				LCD_WriteLineToLCD(LINE2, TEXT_LEFT, "received:  0/0");
				LCD_WriteLineToLCD(LINE4, TEXT_MENU_ITEM, MENU_SUBITEM_BACK);
				lcd_setCursor(0, LINE4);
				Write_char_lcd(LCD_MENU_CURSOR);
				dcf_signalTestProcess = true;
				dcf_signalTestDetection = true;
				if (uart_dcfActive == true)
				{
					uart_writeTestProcess("0","0");
				}
			}
			else if (dcf_signalTestCounter < DCF_SIGNAL_TEST && dcf_signalTestProcess == true)		// count the valid bits an show on LCD
			{
				dcf_signalTestProcess = true;
				dcf_signalTestResult = true;
				dcf_signalTestCounter++;
				if ((dcf_signalTestCounter % 1000)==0)
				{
					char textLine2[16];
					char sendBit_string[3];
					char receivedBit_string[3];
					memset(textLine2, 0, strlen(textLine2));
					uint8_t sendBit = dcf_signalTestCounter/1000;
					sprintf(sendBit_string, "%d", sendBit);
					sprintf(receivedBit_string, "%d", dcf_validSignalCounter);
					strcat(textLine2, "received:  ");
					strcat(textLine2, receivedBit_string);
					strcat(textLine2, "/");
					strcat(textLine2, sendBit_string);
					LCD_Clear();
					LCD_WriteLineToLCD(LINE1, TEXT_LEFT, "wait for signal");
					LCD_WriteLineToLCD(LINE2, TEXT_LEFT, textLine2);
					LCD_WriteLineToLCD(LINE4, TEXT_MENU_ITEM, MENU_SUBITEM_BACK);
					lcd_setCursor(0, LINE4);
					Write_char_lcd(LCD_MENU_CURSOR);
					if (uart_dcfActive == true)
					{
						uart_writeTestProcess(receivedBit_string, sendBit_string);
					}
				}	
			}
			else if (dcf_signalTestCounter == DCF_SIGNAL_TEST && dcf_signalTestProcess == true)		// signal test over, check signal quality
			{
				if (dcf_validSignalCounter < 10)			// less than 10 valit bits received -> bad signal
				{
					dcf_signalTestResult = false;
				}
				else if (dcf_validSignalCounter == 10)		// 10 valid bits received -> signal ok
				{
					dcf_signalTestProcess = false;
				}	

				if (dcf_signalTestResult == false && dcf_showMessageDelay < MESSAGE_DELAY)			// bad signal -> show message on LCD and update for countdown
				{
					if (dcf_showMessageDelay == 0)
					{
						dcf_signalTestDetection = false;
						LCD_Clear();
						LCD_WriteLineToLCD(LINE1, TEXT_LEFT, "bad signal :(");
						LCD_WriteLineToLCD(LINE2, TEXT_LEFT, "please move me");
						LCD_WriteLineToLCD(LINE3, TEXT_LEFT, "somewhere else!");
						LCD_WriteLineToLCD(LINE4, TEXT_LEFT, "I will wait....5");
						if (uart_dcfActive == true)
						{
							uart_writeTestFailed("5");
						}
					}

					dcf_showMessageDelay++;

					if ((dcf_showMessageDelay % 1000)==0)
					{
						dcf_messageDelayCounter--;
						char timeout_string[3];
						char textLine4[16];
						memset(textLine4, 0, strlen(textLine4));
						sprintf(timeout_string, "%d", dcf_messageDelayCounter);
						strcat(textLine4, "I will wait....");
						strcat(textLine4, timeout_string);
						LCD_Clear();
						LCD_WriteLineToLCD(LINE1, TEXT_LEFT, "bad signal :(");
						LCD_WriteLineToLCD(LINE2, TEXT_LEFT, "please move me");
						LCD_WriteLineToLCD(LINE3, TEXT_LEFT, "somewhere else!");
						LCD_WriteLineToLCD(LINE4, TEXT_LEFT, textLine4);
						if (uart_dcfActive == true)
						{
							uart_writeTestFailed(timeout_string);
						}
					}								
				}
				else										// reset signal test values
				{
					dcf_signalTestCounter = 0;
					dcf_validSignalCounter = 0;
					dcf_showMessageDelay = 0;
					dcf_messageDelayCounter = 5;
					dcf_signalTestDetection = true;
				}		
			}
			else if (dcf_signalTestProcess == false)		// signal test successfully finished
			{
				if (dcf_syncCounter == 0 && dcf_synced == false && dcf_invalidSync == false)
				{
					LCD_Clear();
					LCD_WriteLineToLCD(LINE1, TEXT_LEFT, "wait for sync...");
					LCD_WriteLineToLCD(LINE4, TEXT_MENU_ITEM, MENU_SUBITEM_BACK);
					lcd_setCursor(0, LINE4);
					Write_char_lcd(LCD_MENU_CURSOR);
					if (uart_dcfActive == true)
					{
						uart_writeWaitSync();
					}
				}
				dcf_syncCounter++;
			}
		}
   }